#include <stdlib.h>
#include "step_euler.h"

/* Explicit Integrator Step Function: Euler's Method */
int ode_step_euler(ODEFunc f, const ODEData *start, ODEData *end,
                   ODEFloat h, ODESimDataUpdateCB simDataUpdateCB,
                   ODESimDataAllocateCB simDataAllocateCB,
                   ODESimDataFreeCB simDataFreeCB, ODESimDataCopyCB simDataCopyCB,
                   void **_stepData, ODEStepDataFreeCB *stepDataFreeCB) {
    size_t i;
    ODEFloat *f_result = *_stepData;

    /* Allocate our space if needed */
    if(! f_result) {
        f_result = (ODEFloat *)malloc(start->dim * sizeof(ODEFloat));

        /* Check for memory allocation */
        if(!f_result)
            return 2;

        *_stepData = f_result;
        *stepDataFreeCB = free;
    }

    /* Evaluate the differential equation with the current state to get dx/dt */
    f(start, f_result);

    /* Scale the results by delta-t so we get delta-x */
    for(i=0; i < start->dim; i++) {
        end->x[i] = start->x[i] + h * f_result[i];
    }

    /* Update t */
    end->t = start->t + h;

    return 0;
}
